import typing
from TransformsAI.Animo import ObjectInfo, ObjectGroup, GridObject, Directions, VoxelGrid, GridTransform, Actor
from System import IEquatable_1, Array_1, Guid, ValueTuple_2
from TransformsAI.Animo.Constants import TypeIds, Effects, GroundMaterials
from TransformsAI.Animo.Numerics import Vec2Int, Vec3Int
from System.Collections.Generic import List_1, HashSet_1
from TransformsAI.Animo.Intents import Intent, UseIntent, PushIntent
from TransformsAI.Animo.Rewards.Categories import RewardCategory
from TransformsAI.Animo.Rewards import ObjectDestroyedReward, Reward, TargetedReward, RewardTargetTypes, RewardTarget
from TransformsAI.Animo.Tools import ReadOnlyArray_1
from TransformsAI.Animo.Objects.Character import CharacterObject

class LargeSnowballInfo(ObjectInfo, IEquatable_1[LargeSnowballInfo]):
    def __init__(self) -> None: ...
    @property
    def CanStepOnObjects(self) -> bool: ...
    @property
    def CreatesObjects(self) -> Array_1[TypeIds]: ...
    @property
    def IsDestroyedInDeepWater(self) -> bool: ...
    @property
    def IsDestroyedInShallowWater(self) -> bool: ...
    @property
    def IsDestructible(self) -> bool: ...
    @property
    def IsGrabbable(self) -> bool: ...
    @property
    def IsOccupier(self) -> bool: ...
    @property
    def IsUsableOnGrid(self) -> bool: ...
    @property
    def LabelName(self) -> str: ...
    @property
    def MaximumBurnAmount(self) -> int: ...
    @property
    def ObjectGroup(self) -> ObjectGroup: ...
    @property
    def TransformsInto(self) -> Array_1[TypeIds]: ...
    @property
    def Type(self) -> typing.Type[typing.Any]: ...
    @property
    def TypeId(self) -> TypeIds: ...
    @property
    def UsePriority(self) -> float: ...
    def CanOverlap(self, b: TypeIds) -> bool: ...
    def CanReceiveEffect(self, effect: Effects) -> bool: ...
    def CreateObject(self) -> GridObject: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, left: LargeSnowballInfo, right: LargeSnowballInfo) -> bool: ...
    def __ne__(self, left: LargeSnowballInfo, right: LargeSnowballInfo) -> bool: ...
    def ToString(self) -> str: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: LargeSnowballInfo) -> bool:...
        @typing.overload
        def __call__(self, other: ObjectInfo) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class LargeSnowballObject(GridObject):
    def __init__(self) -> None: ...
    Info : ObjectInfo
    @property
    def BurnAmount(self) -> int: ...
    @property
    def Cell(self) -> Vec2Int: ...
    @property
    def Container(self) -> GridObject: ...
    @Container.setter
    def Container(self, value: GridObject) -> GridObject: ...
    @property
    def Direction(self) -> Directions: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def Height(self) -> int: ...
    @property
    def InstanceId(self) -> Guid: ...
    @property
    def IntentsFiredThisTurn(self) -> List_1[Intent]: ...
    @property
    def IsOnGrid(self) -> bool: ...
    @property
    def IsStoodUpon(self) -> bool: ...
    @property
    def MainIntent(self) -> Intent: ...
    @property
    def OriginalInstanceId(self) -> Guid: ...
    @property
    def Position(self) -> Vec3Int: ...
    @property
    def StateObservation(self) -> ValueTuple_2[float, float]: ...
    @property
    def Transform(self) -> GridTransform: ...
    @Transform.setter
    def Transform(self, value: GridTransform) -> GridTransform: ...
    @property
    def TransformedInto(self) -> GridObject: ...
    @property
    def TransformOnGrid(self) -> None: ...
    @TransformOnGrid.setter
    def TransformOnGrid(self, value: typing.Optional[GridTransform]) -> None: ...
    @property
    def TypeId(self) -> TypeIds: ...
    @property
    def UsedIntent(self) -> UseIntent: ...
    @property
    def VariantId(self) -> int: ...
    @VariantId.setter
    def VariantId(self, value: int) -> int: ...
    @property
    def WasDestroyedOrTransformed(self) -> bool: ...
    def TryMove(self, target: GridTransform, outIntents: List_1[Intent], originGroundMaterial: typing.Optional[GroundMaterials] = ...) -> bool: ...


class LargeSnowballPushIntent(UseIntent):
    def __init__(self) -> None: ...
    Causers : HashSet_1[GridObject]
    id : int
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def PushIntent(self) -> PushIntent: ...
    @PushIntent.setter
    def PushIntent(self, value: PushIntent) -> PushIntent: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...
    @property
    def WillContinueUsing(self) -> bool: ...


class LargeSnowballRewards(RewardCategory):
    def __init__(self) -> None: ...
    Destroyed : ObjectDestroyedReward
    name : str
    PushAway : PushLargeSnowballAwayReward
    PushOnto : PushLargeSnowballOntoReward
    PushTowards : PushLargeSnowballTowardsReward
    @property
    def AssociatedObject(self) -> typing.Optional[TypeIds]: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def Rewards(self) -> ReadOnlyArray_1[Reward]: ...


class PushLargeSnowballAwayReward(TargetedReward):
    def __init__(self, categoryId: str, rewardId: str) -> None: ...
    @property
    def CanUserTargetSelf(self) -> bool: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def HasTarget(self) -> bool: ...
    @property
    def RewardId(self) -> str: ...
    @property
    def Targets(self) -> Array_1[RewardTargetTypes]: ...
    def EvaluateOnIntentExecute(self, characterObject: CharacterObject, intentCausedByCharacter: Intent, target: RewardTarget) -> bool: ...
    def TargetObjectTypeFilter(self, obj: ObjectInfo) -> bool: ...


class PushLargeSnowballOntoReward(TargetedReward):
    def __init__(self, categoryId: str, rewardId: str) -> None: ...
    @property
    def CanUserTargetSelf(self) -> bool: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def HasTarget(self) -> bool: ...
    @property
    def RewardId(self) -> str: ...
    @property
    def Targets(self) -> Array_1[RewardTargetTypes]: ...
    def EvaluateOnIntentExecute(self, characterObject: CharacterObject, intentCausedByCharacter: Intent, target: RewardTarget) -> bool: ...
    def TargetObjectTypeFilter(self, obj: ObjectInfo) -> bool: ...


class PushLargeSnowballTowardsReward(TargetedReward):
    def __init__(self, categoryId: str, rewardId: str) -> None: ...
    @property
    def CanUserTargetSelf(self) -> bool: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def HasTarget(self) -> bool: ...
    @property
    def RewardId(self) -> str: ...
    @property
    def Targets(self) -> Array_1[RewardTargetTypes]: ...
    def EvaluateOnIntentExecute(self, characterObject: CharacterObject, intentCausedByCharacter: Intent, target: RewardTarget) -> bool: ...

